#!/bin/bash
# Version 28.0-33_a495bb2b4

#
# Setup script for linux / iptables based PrivX router.
#
# When called as "setup.sh up", this script sets up two custom iptables chains,
# adds jump rules for them to FORWARD chain, and enables IP forwarding.
#
# When called as "setup.sh down", this script removes the custom iptables chains
# and the jump rules for them in FORWARD chain.
#
# When called as "setup.sh list", this script displays jump rules to custom
# iptables chains and rules in the custom chains.
#
# The following variables are read from $ENV_FILE (by default
# "/opt/privx/etc/privx-router.env"):
#   IPTABLES:
#   I6TABLES:
#     Iptables command, defaults "/usr/bin/sudo /usr/sbin/iptables" and
#     "/usr/bin/sudo /usr/sbin/ipt6ables"
#   RAC_IP_POOL:
#   RAC_IP6_POOL:
#     Remote access client IP address pools as space limited list of CIDRs.
#     If not specified then firewalling functionality for the respective IP
#     version is disabled.
#   RAC_BYPASS_TARGETS:
#   RAC_BYPASS6_TARGETS:
#     Remote access client bypass target addresses as space limited list of
#     IP addresses or CIDRs
#   CHAIN_PRIVX_FORWARD_TARGET:
#   CHAIN_PRIVX6_FORWARD_TARGET:
#     Name of custom iptables chain for client to target rules, defaults
#     "PRIVX_FORWARD_TARGET" and "PRIVX6_FORWARD_TARGET"
#   CHAIN_PRIVX_FORWARD_CLIENT:
#   CHAIN_PRIVX6_FORWARD_CLIENT:
#     Name of custom iptables chain for target to client rules, defaults
#     "PRIVX_FORWARD_CLIENT" and "PRIVX6_FORWARD_CLIENT"
#   CHAIN_PRIVX_NAT_PREROUTING:
#   CHAIN_PRIVX6_NAT_PREROUTING:
#     Name of custom iptables chain for target to client DNAT rules, defaults
#     "PRIVX_NAT_PREROUTING" and "PRIVX6_NAT_PREROUTING"
#   CHAIN_PRIVX_NAT_POSTROUTING:
#   CHAIN_PRIVX6_NAT_POSTROUTING:
#     Name of custom iptables chain for target to client SNAT rules, defaults
#     "PRIVX_NAT_POSTROUTING" and "PRIVX6_NAT_POSTROUTING"
#   SYSCTL:
#     Sysctl command, default "/usr/bin/sudo /sbin/sysctl"
#

function up4 {
    # Setup chain for client to target rules
    if ! $IPTABLES -L $CHAIN_PRIVX_FORWARD_TARGET 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX_FORWARD_TARGET"
        $IPTABLES -N $CHAIN_PRIVX_FORWARD_TARGET
    fi

    for ip in $RAC_BYPASS_TARGETS; do
        rulespec="-d $ip -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j ACCEPT"
        if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_TARGET $rulespec 2> /dev/null; then
            echo "Adding bypass accept rule to $CHAIN_PRIVX_FORWARD_TARGET"
            $IPTABLES -A $CHAIN_PRIVX_FORWARD_TARGET $rulespec
        fi
    done

    rulespec="-p tcp -j REJECT --reject-with tcp-reset"
    if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_TARGET $rulespec 2> /dev/null; then
        echo "Adding default tcp reject rule to $CHAIN_PRIVX_FORWARD_TARGET"
        $IPTABLES -A $CHAIN_PRIVX_FORWARD_TARGET $rulespec
    fi

    rulespec="-j REJECT"
    if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_TARGET $rulespec 2> /dev/null; then
        echo "Adding default reject rule to $CHAIN_PRIVX_FORWARD_TARGET"
        $IPTABLES -A $CHAIN_PRIVX_FORWARD_TARGET $rulespec
    fi

    # Setup chain for target to client rules
    if ! $IPTABLES -L $CHAIN_PRIVX_FORWARD_CLIENT 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX_FORWARD_CLIENT"
        $IPTABLES -N $CHAIN_PRIVX_FORWARD_CLIENT
    fi

    for ip in $RAC_BYPASS_TARGETS; do
        rulespec="-s $ip -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j ACCEPT"
        if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_CLIENT $rulespec 2> /dev/null; then
            echo "Adding bypass accept rule to $CHAIN_PRIVX_FORWARD_CLIENT"
            $IPTABLES -A $CHAIN_PRIVX_FORWARD_CLIENT $rulespec
        fi
    done

    rulespec="-p tcp -j REJECT --reject-with tcp-reset"
    if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_CLIENT $rulespec 2> /dev/null; then
        echo "Adding default tcp reject rule to $CHAIN_PRIVX_FORWARD_CLIENT"
        $IPTABLES -A $CHAIN_PRIVX_FORWARD_CLIENT $rulespec
    fi

    rulespec="-j REJECT"
    if ! $IPTABLES -C $CHAIN_PRIVX_FORWARD_CLIENT $rulespec 2> /dev/null; then
        echo "Adding default reject rule to $CHAIN_PRIVX_FORWARD_CLIENT"
        $IPTABLES -A $CHAIN_PRIVX_FORWARD_CLIENT $rulespec
    fi

    # Setup chain for DNAT rules
    if ! $IPTABLES -t nat -L $CHAIN_PRIVX_NAT_PREROUTING 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX_NAT_PREROUTING"
        $IPTABLES -t nat -N $CHAIN_PRIVX_NAT_PREROUTING
    fi

    # Setup chain for SNAT rules
    if ! $IPTABLES -t nat -L $CHAIN_PRIVX_NAT_POSTROUTING 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX_NAT_POSTROUTING"
        $IPTABLES -t nat -N $CHAIN_PRIVX_NAT_POSTROUTING
    fi

    # Setup jump rules
    for cidr in $RAC_IP_POOL; do
        if ! $IPTABLES -C FORWARD -s $cidr -j $CHAIN_PRIVX_FORWARD_TARGET 2> /dev/null; then
            echo "Adding jump rule to FORWARD chain for source $cidr target $CHAIN_PRIVX_FORWARD_TARGET"
            $IPTABLES -A FORWARD -s $cidr -j $CHAIN_PRIVX_FORWARD_TARGET
        fi

        if ! $IPTABLES -C FORWARD -d $cidr -j $CHAIN_PRIVX_FORWARD_CLIENT 2> /dev/null; then
            echo "Adding jump rule to FORWARD chain for destination $cidr target $CHAIN_PRIVX_FORWARD_CLIENT"
            $IPTABLES -A FORWARD -d $cidr -j $CHAIN_PRIVX_FORWARD_CLIENT
        fi

        if ! $IPTABLES -t nat -C PREROUTING -s $cidr -j $CHAIN_PRIVX_NAT_PREROUTING 2> /dev/null; then
            echo "Adding jump rule to nat PREROUTING chain for source $cidr target $CHAIN_PRIVX_NAT_PREROUTING"
            $IPTABLES -t nat -A PREROUTING -s $cidr -j $CHAIN_PRIVX_NAT_PREROUTING
        fi

        if ! $IPTABLES -t nat -C POSTROUTING -s $cidr -j $CHAIN_PRIVX_NAT_POSTROUTING 2> /dev/null; then
            echo "Adding jump rule to nat POSTROUTING chain for source $cidr target $CHAIN_PRIVX_NAT_POSTROUTING"
            $IPTABLES -t nat -A POSTROUTING -s $cidr -j $CHAIN_PRIVX_NAT_POSTROUTING
        fi
    done

    # Enable IP forwarding
    echo "Enabling IP forwarding:"
    $SYSCTL -w net.ipv4.ip_forward=1
}

function down4 {
    # Remove jump rules
    for cidr in $RAC_IP_POOL; do
        echo "Removing jump rule from FORWARD chain for source $cidr target $CHAIN_PRIVX_FORWARD_TARGET"
        $IPTABLES -D FORWARD -s $cidr -j $CHAIN_PRIVX_FORWARD_TARGET || true

        echo "Removing jump rule from FORWARD chain for destination $cidr target $CHAIN_PRIVX_FORWARD_CLIENT"
        $IPTABLES -D FORWARD -d $cidr -j $CHAIN_PRIVX_FORWARD_CLIENT || true

        echo "Removing jump rule from nat PREROUTING chain for source $cidr target $CHAIN_PRIVX_NAT_PREROUTING"
        $IPTABLES -t nat -D PREROUTING -s $cidr -j $CHAIN_PRIVX_NAT_PREROUTING || true

        echo "Removing jump rule from nat POSTROUTING chain for source $cidr target $CHAIN_PRIVX_NAT_POSTROUTING"
        $IPTABLES -t nat -D POSTROUTING -s $cidr -j $CHAIN_PRIVX_NAT_POSTROUTING || true
    done

    # Flush and delete client to target chain
    echo "Deleting chain $CHAIN_PRIVX_FORWARD_TARGET"
    $IPTABLES -F $CHAIN_PRIVX_FORWARD_TARGET || true
    $IPTABLES -X $CHAIN_PRIVX_FORWARD_TARGET || true

    # Flush and delete target to client chain
    echo "Deleting chain $CHAIN_PRIVX_FORWARD_CLIENT"
    $IPTABLES -F $CHAIN_PRIVX_FORWARD_CLIENT || true
    $IPTABLES -X $CHAIN_PRIVX_FORWARD_CLIENT || true

    # Flush and delete DNAT chain
    echo "Deleting chain $CHAIN_PRIVX_NAT_PREROUTING"
    $IPTABLES -t nat -F $CHAIN_PRIVX_NAT_PREROUTING || true
    $IPTABLES -t nat -X $CHAIN_PRIVX_NAT_PREROUTING || true

    # Flush and delete SNAT chain
    echo "Deleting chain $CHAIN_PRIVX_NAT_POSTROUTING"
    $IPTABLES -t nat -F $CHAIN_PRIVX_NAT_POSTROUTING || true
    $IPTABLES -t nat -X $CHAIN_PRIVX_NAT_POSTROUTING || true  
}

function list4 {
    echo "Jump rules:"
    $IPTABLES -nvL FORWARD | grep -E "$CHAIN_PRIVX_FORWARD_TARGET|$CHAIN_PRIVX_FORWARD_CLIENT"

    echo "Client to target chain $CHAIN_PRIVX_FORWARD_TARGET:"
    $IPTABLES -nvL $CHAIN_PRIVX_FORWARD_TARGET

    echo "Target to client chain $CHAIN_PRIVX_FORWARD_CLIENT:"
    $IPTABLES -nvL $CHAIN_PRIVX_FORWARD_CLIENT

    echo "DNAT chain $CHAIN_PRIVX_NAT_PREROUTING:"
    $IPTABLES -t nat -nvL $CHAIN_PRIVX_NAT_PREROUTING

    echo "SNAT chain $CHAIN_PRIVX_NAT_POSTROUTING:"
    $IPTABLES -t nat -nvL $CHAIN_PRIVX_NAT_POSTROUTING
}

function up6 {
    # Setup chain for client to target rules
    if ! $IP6TABLES -L $CHAIN_PRIVX6_FORWARD_TARGET 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX6_FORWARD_TARGET"
        $IP6TABLES -N $CHAIN_PRIVX6_FORWARD_TARGET
    fi

    for ip in $RAC_BYPASS6_TARGETS; do
        rulespec="-d $ip -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j ACCEPT"
        if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_TARGET $rulespec 2> /dev/null; then
            echo "Adding bypass accept rule to $CHAIN_PRIVX6_FORWARD_TARGET"
            $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_TARGET $rulespec
        fi
    done

    rulespec="-p tcp -j REJECT --reject-with tcp-reset"
    if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_TARGET $rulespec 2> /dev/null; then
        echo "Adding default tcp reject rule to $CHAIN_PRIVX6_FORWARD_TARGET"
        $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_TARGET $rulespec
    fi

    rulespec="-j REJECT"
    if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_TARGET $rulespec 2> /dev/null; then
        echo "Adding default reject rule to $CHAIN_PRIVX6_FORWARD_TARGET"
        $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_TARGET $rulespec
    fi

    # Setup chain for target to client rules
    if ! $IP6TABLES -L $CHAIN_PRIVX6_FORWARD_CLIENT 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX6_FORWARD_CLIENT"
        $IP6TABLES -N $CHAIN_PRIVX6_FORWARD_CLIENT
    fi

    for ip in $RAC_BYPASS6_TARGETS; do
        rulespec="-s $ip -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j ACCEPT"
        if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec 2> /dev/null; then
            echo "Adding bypass accept rule to $CHAIN_PRIVX6_FORWARD_CLIENT"
            $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec
        fi
    done

    rulespec="-p tcp -j REJECT --reject-with tcp-reset"
    if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec 2> /dev/null; then
        echo "Adding default tcp reject rule to $CHAIN_PRIVX6_FORWARD_CLIENT"
        $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec
    fi

    rulespec="-j REJECT"
    if ! $IP6TABLES -C $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec 2> /dev/null; then
        echo "Adding default reject rule to $CHAIN_PRIVX6_FORWARD_CLIENT"
        $IP6TABLES -A $CHAIN_PRIVX6_FORWARD_CLIENT $rulespec
    fi

    # Setup chain for DNAT rules
    if ! $IP6TABLES -t nat -L $CHAIN_PRIVX6_NAT_PREROUTING 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX6_NAT_PREROUTING"
        $IP6TABLES -t nat -N $CHAIN_PRIVX6_NAT_PREROUTING
    fi

    # Setup chain for SNAT rules
    if ! $IP6TABLES -t nat -L $CHAIN_PRIVX6_NAT_POSTROUTING 2> /dev/null; then
        echo "Creating chain $CHAIN_PRIVX6_NAT_POSTROUTING"
        $IP6TABLES -t nat -N $CHAIN_PRIVX6_NAT_POSTROUTING
    fi

    # Setup jump rules
    for cidr in $RAC_IP6_POOL; do
        if ! $IP6TABLES -C FORWARD -s $cidr -j $CHAIN_PRIVX6_FORWARD_TARGET 2> /dev/null; then
            echo "Adding jump rule to FORWARD chain for source $cidr target $CHAIN_PRIVX6_FORWARD_TARGET"
            $IP6TABLES -A FORWARD -s $cidr -j $CHAIN_PRIVX6_FORWARD_TARGET
        fi

        if ! $IP6TABLES -C FORWARD -d $cidr -j $CHAIN_PRIVX6_FORWARD_CLIENT 2> /dev/null; then
            echo "Adding jump rule to FORWARD chain for destination $cidr target $CHAIN_PRIVX6_FORWARD_CLIENT"
            $IP6TABLES -A FORWARD -d $cidr -j $CHAIN_PRIVX6_FORWARD_CLIENT
        fi

        if ! $IP6TABLES -t nat -C PREROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_PREROUTING 2> /dev/null; then
            echo "Adding jump rule to nat PREROUTING chain for source $cidr target $CHAIN_PRIVX6_NAT_PREROUTING"
            $IP6TABLES -t nat -A PREROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_PREROUTING
        fi

        if ! $IP6TABLES -t nat -C POSTROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_POSTROUTING 2> /dev/null; then
            echo "Adding jump rule to nat POSTROUTING chain for source $cidr target $CHAIN_PRIVX6_NAT_POSTROUTING"
            $IP6TABLES -t nat -A POSTROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_POSTROUTING
        fi
    done

    # Enable IPv6 forwarding
    echo "Enabling IPv6 forwarding:"
    $SYSCTL -w net.ipv6.conf.all.forwarding=1
}

function down6 {
    # Remove jump rules
    for cidr in $RAC_IP6_POOL; do
        echo "Removing jump rule from FORWARD chain for source $cidr target $CHAIN_PRIVX6_FORWARD_TARGET"
        $IP6TABLES -D FORWARD -s $cidr -j $CHAIN_PRIVX6_FORWARD_TARGET || true

        echo "Removing jump rule from FORWARD chain for destination $cidr target $CHAIN_PRIVX6_FORWARD_CLIENT"
        $IP6TABLES -D FORWARD -d $cidr -j $CHAIN_PRIVX6_FORWARD_CLIENT || true

        echo "Removing jump rule from nat PREROUTING chain for source $cidr target $CHAIN_PRIVX6_NAT_PREROUTING"
        $IP6TABLES -t nat -D PREROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_PREROUTING || true

        echo "Removing jump rule from nat POSTROUTING chain for source $cidr target $CHAIN_PRIVX6_NAT_POSTROUTING"
        $IP6TABLES -t nat -D POSTROUTING -s $cidr -j $CHAIN_PRIVX6_NAT_POSTROUTING || true
    done

    # Flush and delete client to target chain
    echo "Deleting chain $CHAIN_PRIVX6_FORWARD_TARGET"
    $IP6TABLES -F $CHAIN_PRIVX6_FORWARD_TARGET || true
    $IP6TABLES -X $CHAIN_PRIVX6_FORWARD_TARGET || true

    # Flush and delete target to client chain
    echo "Deleting chain $CHAIN_PRIVX6_FORWARD_CLIENT"
    $IP6TABLES -F $CHAIN_PRIVX6_FORWARD_CLIENT || true
    $IP6TABLES -X $CHAIN_PRIVX6_FORWARD_CLIENT || true

    # Flush and delete DNAT chain
    echo "Deleting chain $CHAIN_PRIVX6_NAT_PREROUTING"
    $IP6TABLES -t nat -F $CHAIN_PRIVX6_NAT_PREROUTING || true
    $IP6TABLES -t nat -X $CHAIN_PRIVX6_NAT_PREROUTING || true

    # Flush and delete SNAT chain
    echo "Deleting chain $CHAIN_PRIVX6_NAT_POSTROUTING"
    $IP6TABLES -t nat -F $CHAIN_PRIVX6_NAT_POSTROUTING || true
    $IP6TABLES -t nat -X $CHAIN_PRIVX6_NAT_POSTROUTING || true
}

function list6 {
    echo "IPv6 Jump rules:"
    $IP6TABLES -nvL FORWARD | grep -E "$CHAIN_PRIVX6_FORWARD_TARGET|$CHAIN_PRIVX6_FORWARD_CLIENT"

    echo "IPv6 Client to target chain $CHAIN_PRIVX6_FORWARD_TARGET:"
    $IP6TABLES -nvL $CHAIN_PRIVX6_FORWARD_TARGET

    echo "IPv6 Target to client chain $CHAIN_PRIVX6_FORWARD_CLIENT:"
    $IP6TABLES -nvL $CHAIN_PRIVX6_FORWARD_CLIENT

    echo "IPv6 DNAT chain $CHAIN_PRIVX6_NAT_PREROUTING:"
    $IP6TABLES -t nat -nvL $CHAIN_PRIVX6_NAT_PREROUTING

    echo "IPv6 SNAT chain $CHAIN_PRIVX6_NAT_POSTROUTING:"
    $IP6TABLES -t nat -nvL $CHAIN_PRIVX6_NAT_POSTROUTING
}


# Read env variables and set defaults
: "${ENV_FILE:=/opt/privx/etc/privx-router.env}"
if [ ! -r $ENV_FILE ]; then
    echo "env is missing" >&2
    exit 1
fi

source "$ENV_FILE"

: "${SYSCTL:=/usr/bin/sudo /sbin/sysctl}"
if [ -n "$RAC_IP_POOL" ]; then
    : "${IPTABLES:=/usr/bin/sudo /usr/sbin/iptables}"
    : "${CHAIN_PRIVX_FORWARD_TARGET:=PRIVX_FORWARD_TARGET}"
    : "${CHAIN_PRIVX_FORWARD_CLIENT:=PRIVX_FORWARD_CLIENT}"
    : "${CHAIN_PRIVX_NAT_PREROUTING:=PRIVX_NAT_PREROUTING}"
    : "${CHAIN_PRIVX_NAT_POSTROUTING:=PRIVX_NAT_POSTROUTING}"
fi
if [ -n "$RAC_IP6_POOL" ]; then
    : "${IP6TABLES:=/usr/bin/sudo /usr/sbin/ip6tables}"
    : "${CHAIN_PRIVX6_FORWARD_TARGET:=PRIVX6_FORWARD_TARGET}"
    : "${CHAIN_PRIVX6_FORWARD_CLIENT:=PRIVX6_FORWARD_CLIENT}"
    : "${CHAIN_PRIVX6_NAT_PREROUTING:=PRIVX6_NAT_PREROUTING}"
    : "${CHAIN_PRIVX6_NAT_POSTROUTING:=PRIVX6_NAT_POSTROUTING}"
fi

case "$1" in

up)
    if [ -n "$RAC_IP_POOL" ]; then
        up4
    fi
    if [ -n "$RAC_IP6_POOL" ]; then
        up6
    fi
    ;;

down)
    if [ -n "$RAC_IP_POOL" ]; then
        down4
    fi
    if [ -n "$RAC_IP6_POOL" ]; then
        down6
    fi
    ;;

list)
    if [ -n "$RAC_IP_POOL" ]; then
        list4
    fi
    if [ -n "$RAC_IP6_POOL" ]; then
        list6
    fi
    ;;

*)
    echo "usage: $0 up|down|list"
    exit 1
    ;;
esac
